MQSeries' JMS 'publish-and-subscribe'
Chris Markes
HCI Architect
IBM UK Laboratories	c C Markes 2000



#!/bin/ksh

JMS_HOME=/usr/mqm/java
JMS_LIB=$JMS_HOME/lib
JMS_CLASSPATH=/usr/jdk_base/lib/classes.zip:\
$JMS_LIB:\
$JMS_LIB/com.ibm.mqjms.jar:\
$JMS_LIB/jms.jar:\
$JMS_LIB/jndi.jar:\
$JMS_LIB/fscontext.jar:\
$JMS_LIB/ldap.jar:\
$JMS_LIB/providerutil.jar:\
$JMS_LIB/com.ibm.mq.jar:\
$JMS_LIB/com.ibm.mqbind.jar:.
FACTORY=com.sun.jndi.fscontext.RefFSContextFactory
REPOSITORY=file:$JMS_HOME/directory

echo Starting...

java -classpath $JMS_CLASSPATH \
-Djava.naming.factory.initial=$FACTORY \
-Djava.naming.provider.url=$REPOSITORY \
$@


@echo off

SET JMS_HOME=\MQSeries\java
SET JMS_LIB=%JMS_HOME%\lib
SET JMS_CLASSPATH=%JMS_LIB%;%JMS_LIB%\com.ibm.mqjms.jar;%JMS_LIB%\jms.jar;%JM S_LIB%\jndi.jar;%JMS_LIB%\fscontext.jar;%JMS_LIB%\ldap.jar;%JMS_LIB%\prov iderutil.jar;%JMS_LIB%\com.ibm.mq.jar;%JMS_LIB%\com.ibm.mqbind.jar;.
SET FACTORY=com.sun.jndi.fscontext.RefFSContextFactory
SET REPOSITORY=file:/MQSeries/mqm/java/directory

echo Starting...

java -classpath %JMS_CLASSPATH% -Djava.naming.factory.initial=%FACTORY% -Djava.naming.provider.url=%REPOSITORY% %1 %2 %3 %4 %5


import javax.naming.*;
import javax.jms.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/**
 * A simple AWT GUI application demonstrating the basic capabilities of 
 * JMS publish-subscribe. A JMSPager subscribes to a topic that 
 * represents a user and receives messages both synchronously and 
 * asynchronously.
 */

public class JMSPager extends Frame implements 
   MessageListener, ExceptionListener
{
   static InitialContext context;

   final static int COLUMNS = 20;

   Vector vector = new Vector ();

   Checkbox onOffButton;
   TextField messageText;
   Button leftButton;
   Button rightButton;
   Label statusLabel;

   TopicConnection connection;
   TopicSession session;
   Topic topic;
   TopicSubscriber subscriber;

   int index = 0;
   /** 
    * Runs the JMSPager application.
    */ 

   public static void main (String [] args)
   {
     try
      {
         context = new InitialContext (System.getProperties ());

         try
         {
            new JMSPager (args [0]).show ();
         }

         catch (ArrayIndexOutOfBoundsException abe)
         {
            new JMSPager (System.getProperty ("jmspager.user.name", 
               System.getProperty ("user.name"))).show ();
         }
      }

      catch (JMSException jmse)
      {
         System.err.println ("JMS error: " + jmse + 
            " (" + jmse.getLinkedException () + ')');
         System.exit (0);
      }

      catch (NamingException ne)
      {
         System.err.println ("JNDI error: " + ne);
         System.exit (0);
      }

      catch (ArrayIndexOutOfBoundsException abe)
      {
         System.err.println ("Usage: java JMSPager user-name");
      }
   }
  /**
    * Returns a named TopicConnection using the JNDI. The 
    * TopicConnectionFactory is a JMS-administered object.
    */

   static TopicConnection getConnection (String name) throws 
      JMSException, NamingException
   {
      TopicConnectionFactory factory = (TopicConnectionFactory) 
         context.lookup (name);

      return factory.createTopicConnection ();
   }

   /**
    * Returns a named Topic handle using the JNDI. The Topic is a 
    * JMS-administered object.
    */

   static Topic getTopic (String name) throws 
      JMSException, NamingException
   {
      return (Topic) context.lookup (name);
   }

   /**
    * Default constructor. Builds a simple AWT layout and then 
    * establishes a connection to the JMS service provider. The 
    * connection must be explicitly started before messages can 
    * be received.
    */
   public JMSPager (String id) throws 
      JMSException, NamingException
   {
      super ("JMSPager: " + id);

      setLayout (new BorderLayout (6, 2));
      setBackground (Color.lightGray);
      add ("North", onOffButton = new Checkbox ("active", false));
      add ("Center", messageText = new TextField (COLUMNS));
      add ("West", leftButton = new Button (" < "));
      add ("East", rightButton = new Button (" > "));
      add ("South", statusLabel = new Label ());
      pack ();
      setResizable (false);
      messageText.setEditable (false);

      // WindowListener to deal with quitting the app.

      addWindowListener (new WindowAdapter () 
      {
         public void windowClosing (WindowEvent event)
         {
            disconnect ();
            System.exit (0);
         }
      });

      // ActionListener attached to left ('select') button.

      leftButton.addActionListener (new ActionListener ()
      {
         public void actionPerformed (ActionEvent event)
         {
            select ();
         }
      });

      // ActionListener attached to right ('ok') button.

      rightButton.addActionListener (new ActionListener () 
      {
         public void actionPerformed (ActionEvent event)
         {
            display ();
         }
      });

      // Item listener attached to Checkbox ('on/off button').

      onOffButton.addItemListener (new ItemListener ()
      {
         public void itemStateChanged (ItemEvent event)
         {
            if (event.getStateChange () == ItemEvent.SELECTED)
            {
               activate ();
            }
            else
            {
               deactivate ();
            }
         }
      });

      // Establish a connection to the JMS service provider.

      connection = getConnection 
         (System.getProperty ("jmspager.server.name", "jmspager"));
      connection.setExceptionListener (this);
      session = connection.createTopicSession 
         (false, Session.AUTO_ACKNOWLEDGE);
      topic = getTopic (id);
      subscriber = session.createSubscriber (topic);   // non-durable

      // Enable the connection to receive messages.

      connection.start ();
   }
   /**
    * Turns the pager on. First checks synchronously for available 
    * messages (delivered while turned off), then switches to 
    * asynchronous receipt mode by establishing the JMSPager as a 
    * MessageListener. In asynchronous mode, JMS calls the onMessage
    * method when new messages arrive.
    */

   public synchronized void activate ()
   {
      try
      {
         Message message = subscriber.receive (0);
         int old = vector.size ();
         int arrived = 0;

         // Add any received messages to our local store (the Vector).

         while (message != null)
         {
            vector.addElement (message);
            arrived++;
            message = subscriber.receive (0);
         }

         showText (arrived + " new " + 
            (arrived == 1 ? "message" : "messages") + 
            " (" + old + " old)");

         // Switch to asynchronous mode. 
         // 
         // Note: after attaching a message listener, attempts to 
         // receive synchronously on the same session would fail.

         subscriber.setMessageListener (this);
      }

      catch (JMSException jmse)
      {
         showError (jmse);
      }
   }
   /**
    * Turns the pager off by disconnecting the message listener on the 
    * session. The subscription and connection to the JMS service 
    * provider is retained.
    */

   public synchronized void deactivate ()
   {
      try
      {
         // Remove the message listener.

         subscriber.setMessageListener (null);

         showText ("");
         index = -1;
      }

      catch (JMSException jmse)
      {
         showError (jmse);
      }
   }
   /**
    * Steps through the message selection menu.
    */

   public synchronized void select ()
   {
      if (vector.size () == 0)
      {
         showText ("No messages.");
      }
      else if (index < vector.size () - 1)
      {
         showText ("Message " + (++index + 1) + '?');
      }
      else
      {
         showText ("Erase all?");
         index = -1;
      }
   }

   /**
    * Displays the message from the local store (the Vector), as 
    * identified by the current index (a negative index means 
    * 'erase all messages'). 
    */
   public synchronized void display ()
   {
      if (vector.size () == 0)
      {
         showText ("No messages.");
      }
      else if (index < 0)
      {
         showText (vector.size () + " messages erased.");
         vector.removeAllElements ();
      }
      else
      {
         try
         {
            scroll ((TextMessage) vector.elementAt (index++));
         }

         catch (ArrayIndexOutOfBoundsException abe)
         {
            showText ("No more messages.");
         }

         catch (JMSException jmse)
         {
            showError (jmse);
         }
      }
   }
   /**
    * Displays the specified message content by scrolling across the 
    * text field.
    */

   public synchronized void scroll (TextMessage message) throws 
      JMSException
   {
      String text = index + ": " + message.getText ().trim ();

      showText (text);

      for (int i = COLUMNS; i < text.length (); i++)
      {
         messageText.setCaretPosition (i);
   
         try
         {
            Thread.sleep (200);
         }

         catch (InterruptedException ie)
         {
            ;
         }
      }
   }
   /**
    * MessageListener interface method for asynchronous message 
    * delivery. The pager application is passed newly arrived 
    * messages by the invocation of this method.
    */

   public synchronized void onMessage (Message message)
   {
      System.out.print ((char) 0x07);   // beep
      vector.addElement (message);
      showText ("New message!");
      index = vector.size () - 1;
   }
   /**
    * ExceptionListener interface method for asynchronous exception 
    * notification.
    */

   public void onException (JMSException e)
   {
      showError (e);
   }
   /**
    * Drops the JMS service provider connection.
    */

   public synchronized void disconnect ()
   {
      try
      {
         subscriber.setMessageListener (null);
         subscriber.close ();
         session.close ();
         connection.close ();
      }

      catch (JMSException jmse)
      {
         ;
      }
   }
   /**
    * Convenience method for displaying text in the central text field.
    */

   synchronized void showText (String text)
   {
      messageText.setText (text);
   }

   /**
    * Convenience method for displaying a message in the status area.
    */

   void showStatus (String message)
   {
      statusLabel.setText (message);
   }

   /**
    * Convenience method for displaying an error.
    */

   void showError (Exception e)
   {
      if (e instanceof JMSException)
      {
         System.err.println (e + 
            " (" + ((JMSException) e).getLinkedException () + ')');
      }
      else
      {
         System.err.println (e);
      }

      showStatus (e.getMessage ());
   }
   /**
    * Container method establishes a border margin (cosmetic).
    */

   public Insets getInsets ()
   {
      Insets insets = super.getInsets ();

      return new Insets (insets.top + 6, insets.left + 6, 
         insets.bottom, insets.right + 6);
   }
}


